"""Gateway connecting to ATAG thermostat."""
from pyatag.const import *
from pyatag.helpers import *
from pyatag.errors import RequestError, ResponseError

import json
import asyncio, aiohttp
import logging

_LOGGER = logging.getLogger(__name__)

class atagDataStore:

    def __init__(
            self, session=None, host=None, port=DEFAULT_PORT, mail=None, interface=DEFAULT_INTERFACE, scan_interval=DEFAULT_SCAN_INTERVAL, sensors=DEFAULT_SENSORS):
        
        self.host_data = HostData(host=host, port=port, interface=interface, mail=mail)
        if session is None:
            session = aiohttp.ClientSession()
        if type(session).__name__ == 'ClientSession':
            self._connector = HttpConnector(self.host_data, session)
        else:
            _LOGGER.error("Not a valid session: %s", type(session).__name__ )

        self.scan_interval = scan_interval
        self.data = {}
        self.sensors = sensors
        self.sensordata = {}
        self.paired = False

    async def async_update(self):
        """Read data from thermostat."""
        try:
            self.data = await self._connector.atag_put(data=self.host_data.retrieve_msg, path=RETRIEVE_PATH)
            #_LOGGER.debug("Retrieve reply: %s", self.data)
            self.sensordata = self.store_sensor_data()
            _LOGGER.debug("Atag sensordata updated:\n %s", self.sensordata)
        except json.JSONDecodeError as err:
            raise ResponseError("Unable to decode Json response : {}".
                                format(err))

    async def async_set_atag(self, _target_mode=None, _target_temp=None):
        jsonPayload = self.host_data.get_update_msg(_target_mode, _target_temp)
        _LOGGER.debug("Updating: Mode:[%s], Temp:[%s]", _target_mode, _target_temp)
        try:
            json_data = await self._connector.atag_put(data=jsonPayload, path=UPDATE_PATH)
            _LOGGER.debug("Update reply: %s", json_data)
            return json_data[UPDATE_REPLY][ACC_STATUS] == 2
        except json.JSONDecodeError as err:
            raise ResponseError("Unable to decode Json response : {}".
                                format(err))

    def store_sensor_data(self):
        result = {}
        try:
            status = self.data[RETRIEVE_REPLY][REPORT]
            _LOGGER.debug("status: %s", status)
            for sensor in self.sensors:
                datafield = SENSOR_TYPES[sensor][3]
                if sensor == BOILER_STATUS:
                    s = int(status[sensor])
                    result[sensor] = BOILER_STATES[s & 14]
                elif DETAILS in status and datafield in status[DETAILS]:
                    result[sensor] = float(status[DETAILS][datafield])
                elif datafield in status:
                    result[sensor] = float(status[datafield])
                else:
                    _LOGGER.error('Atag sensor %s type error: %s', sensor, datafield)
                    return False
        except:
            _LOGGER.warn('Atag sensor failed to update: %s', self.data)
            return False
        operation_mode_int = self.data[RETRIEVE_REPLY][CONTROL][ATTR_OPERATION_MODE_INT]
        result[ATTR_OPERATION_MODE] = INT_MODES[operation_mode_int]
        result[ATTR_REPORT_TIME] = get_time_from_stamp(status[ATTR_REPORT_TIME])
        return result

    async def async_check_pair_status(self):
        if self.paired:
            return True
        try:
            json_data = await self._connector.atag_put(data=self.host_data.pair_msg, path=PAIR_PATH)
            status = json_data[PAIR_REPLY][ACC_STATUS]
            _LOGGER.debug("AtagDataStore pairing\n%s\n%s",self.host_data.pair_msg,json_data)
        except:
            _LOGGER.error("Pairing failed\n%s\n%s",self.host_data.pair_msg, self.host_data.baseurl)
            return False
        if status == 2:
            self.paired = True
            _LOGGER.debug("AtagDataStore paired")
            return self.paired
        elif status == 1:
            print("Waiting for pairing confirmation")
        elif status == 3:
            print("Waiting for pairing confirmation")
        elif status == 0:
            print("No status returned from ATAG One")
        _LOGGER.warning("Atag not paired!\n%s", json_data)
        self.paired = False
        return self.paired


from datetime import datetime, timezone, timedelta
from aiohttp import client_exceptions
from asyncio import TimeoutError
from numbers import Number
import json

from pyatag.const import REQUEST_INFO, MODES, INT_MODES, HTTP_HEADER, DEFAULT_TIMEOUT, DEFAULT_INTERFACE, DEFAULT_PORT
from pyatag.errors import AtagException, RequestError, ResponseError, Response404Error

MAC = 'mac'
HOSTNAME = 'hostname'
MAIL = 'email'
STATE_UNKNOWN = 'unknown'
URL = 'url'

def get_host_data(host=None, port=DEFAULT_PORT, interface=DEFAULT_INTERFACE, mail=None):
    if host is None:
        raise AtagException("Invalid/None host data provided")
    import netifaces, socket
    data = {
        URL: ''.join(['http://', str(host), ':', str(port),'/']),
        MAC: netifaces.ifaddresses(interface)[
                netifaces.AF_LINK][0]['addr'].upper(),
        HOSTNAME: socket.gethostname(),
        MAIL: mail
    }
    return data


def get_int_from_mode(mode):
    return MODES[mode]

def get_mode_from_int(int_mode):
    if int_mode in INT_MODES:
        return INT_MODES[int_mode]
    else:
        return STATE_UNKNOWN

def get_hostname():
    import socket
    return socket.gethostname()
    
def get_time_from_stamp(secs_after_2k):
    return datetime(2000, 1, 1, tzinfo=timezone.utc) + timedelta(seconds = secs_after_2k)

class HttpConnector:
    """HTTP connector to Bosch thermostat."""

    def __init__(self, hostdata, websession):
        """Init of HTTP connector."""
        self.hostdata = hostdata
        self._websession = websession
        self._request_timeout = DEFAULT_TIMEOUT 
       
    async def atag_put(self, data, path):
        """Make a put request to the API."""
        
        posturl = ''.join([str(self.hostdata.baseurl), str(path)])
        try:
            async with self._websession.put(
                    posturl,
                    json=data,
                    headers=HTTP_HEADER,
                    timeout=self._request_timeout) as req:
                data = await req.text()
                json_result = json.loads(data)
                return json_result
        except (client_exceptions.ClientError, TimeoutError) as err:
            raise ResponseError('Error putting data Atag: %s', err)
        except json.JSONDecodeError as jsonerr:
            raise ResponseError('Unable to decode Json response: %s', jsonerr)

    def set_timeout(self, timeout=DEFAULT_TIMEOUT):
        """Set timeout for API calls."""
        self._request_timeout = timeout

class HostData:
    def __init__(self, host=None, port=DEFAULT_PORT, interface=DEFAULT_INTERFACE, mail=None):
        """Connection info store."""
        if host is None:
            raise AtagException("Invalid/None host data provided")
        import netifaces, socket
        self.ataghost = host
        self.hostname = socket.gethostname()
        self.port = port
        self.baseurl = ''.join(['http://', str(host), ':', str(port), '/'])
        self.interface = interface
        self.email = mail
        self.mac = netifaces.ifaddresses(interface)[
                     netifaces.AF_LINK][0]['addr'].upper()
        self.set_pair_msg()
        self.set_retrieve_msg()

    def set_retrieve_msg(self):
        jsonPayload = {
            "retrieve_message": {
                "seqnr": 1,
                "account_auth": {
                    'user_account': self.email,
                    'mac_address': self.mac
                },
                "info": REQUEST_INFO
            }
        }
        self.retrieve_msg = jsonPayload

    def set_pair_msg(self):
        jsonPayload = {
            "pair_message": {
                "seqnr": 1,
                "account_auth": {
                    'user_account': self.email,
                    'mac_address': self.mac
                },
                "accounts": {
                    "entries": [
                        {
                            "user_account": self.email,
                            "mac_address": self.mac,
                            "device_name": self.hostname,
                            "account_type": 1
                        }
                    ]
                }
            }
        }
        self.pair_msg = jsonPayload
    
    def get_update_msg(self, _target_mode=None, _target_temp=None):
        _target_mode_int = None
        if _target_mode is None and _target_temp is None:
            raise RequestError("No update data received")
#            elif _target_mode is not None and _target_temp is not None:
#                raise RequestError("Cannot update mode and temperature simultaneously")
        elif _target_mode is not None:
            if _target_mode in MODES:
                _target_mode_int = MODES[_target_mode]
            else:
                raise RequestError("Invalid update mode: %s", _target_mode)
        elif _target_temp is not None and not isinstance(_target_temp, Number):
            raise RequestError("Not a valid temperature: %s", _target_temp)
        
        jsonPayload = {
            'update_message': {
                'seqnr': 1,
                'account_auth': {
                    'user_account': self.email,
                    'mac_address': self.mac
                },
                'control': {
                    'ch_mode': _target_mode_int,
                    'ch_mode_temp': _target_temp
                }
            }
        }
        return(jsonPayload)


import aiohttp
import asyncio
from pyatag.gateway import atagDataStore


async def test(TESTDATA):

    async with aiohttp.ClientSession() as session:
        atag = atagDataStore(host=TESTDATA["_host"],
                             port=TESTDATA["_port"],
                             mail=TESTDATA["_mail"],
                             interface=TESTDATA["_interface"],
                             scan_interval=TESTDATA["_scan_interval"],
                             session=session,
                             sensors=TESTDATA["_sensors"])
        await atag.async_check_pair_status()
        # await atag.async_update()
        print(await atag.async_set_atag(_target_mode="manual", _target_temp=13))
        return atag

asyncio.get_event_loop().run_until_complete(test(TESTDATA))



TESTDATA = {
    "_host": "", #atag IP
    "_port": 10000,
    "_mail": "", #email registered in portal
    "_scan_interval": 30,
    "_interface": "en0", #interface on which API runs
    "_sensors": [ #sensors to test, see const.py
        "temperature",
        "current_temp"
    ]
}


from pyatag.const import *
from pyatag.helpers import *
from pyatag.errors import *

import json
import asyncio 
import aiohttp
from datetime import timedelta

from pyatag.gateway import atagDataStore
"""
from pyatag.input import TESTDATA #put real data

async def main():

    async with aiohttp.ClientSession() as session:
        atag =  atagDataStore(host=TESTDATA["_host"],
                              port=TESTDATA["_port"],
                              mail=TESTDATA["_mail"],
                              interface=TESTDATA["_interface"],
                              scan_interval=TESTDATA["_scan_interval"],
                              session=session,
                              sensors=TESTDATA["_sensors"])
        await atag.async_check_pair_status()
        #await atag.async_update()
        print(await atag.async_set_atag(_target_mode="manual",_target_temp=13))
        return atag

asyncio.get_event_loop().run_until_complete(main())

"""



from distutils.core import setup
setup(
    name='pyatag',         # How you named your package folder (MyLib)
    packages=['pyatag'],   # Chose the same as "name"
    version='0.1.4',      # Start with a small number and increase it with every change you make
    # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    license='MIT',
    description='Python API to connect ATAG One Thermostat',
    author='Mats',                   # Type in your name
    author_email='mats.nelissen@gmail.com',      # Type in your E-Mail
    url='https://github.com/MatsNl/pyatag',
    #  download_url = 'https://github.com/user/reponame/archive/v_01.tar.gz',    # I explain this later on
    #  keywords = ['SOME', 'MEANINGFULL', 'KEYWORDS'],   # Keywords that define your package best
    install_requires=[
        'asyncio',
        'aiohttp',
        'netifaces'
    ],
    classifiers=[
        # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Development Status :: 3 - Alpha',
        #    'Intended Audience :: Developers',      # Define that your audience are developers
        #    'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',   # Again, pick a license
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)
